//==========================================================================
// MLIGHTS.CPP     Source file for the main processing function and the main
//                 window message handling function of the MLIGHTS.EXE 
//                 application.
//
// CREATED BY      George C Cross, for Synergy Solutions Inc
// CREATED ON      August 5, 1995
// LAST MODIFIED   October 3, 1995.  Final build for beta release.
//
// (c) Copyright Synergy Solutions Inc, 1995. All rights reserved.
//========================================================================== 

#include "mlights.h"    //Header for this source file
#include "resource.h"   //Header for all resource identifiers
#include "prochook.h"   //Header for the PROCHOOK.DLL library
#include "mainwnd.h"    //Header for main window message response functions
#include "cshape.h"     //Header for the CSHAPE class (emulates modem lights)
#include "clabel.h"     //Header for the CLABEL class (displays light labels)
#include <string.h>     //Header for string comparison functions
#include <stdlib.h>     //Header for string conversion functions
#include <commdlg.h>    //Header for Windows API common dialog constants
#include "ctl3d.h"      //Header for Microsoft 3D controls


//==========================================================================
// Below are arrays of constants used to reduce the amount code when
// accomplishing various initialization tasks. 
//
// *** NOTE: ****
// While his technique assists greatly in reducing amount of code written,
// it can be rather confusing AND the order of entries in each array is
// CRUCIAL !!!
//==========================================================================

//
//Array used to create labels for the lights
//
char LabelConstants[MAX_NUM_LIGHTS][MAX_LABEL_CHARS] = {"CTS",
                                                        "DSR",
                                                        "RI",
                                                        "CD",
                                                        "TX",
                                                        "RX" };
//
//Array used to read and write MLIGHTS.INI
//                                                   
char OptionEntryStrings[NUM_OPTIONS][20] = {"MinIfPortClosed",
                                            "TopMost",
                                            "HideCaption"};
//
//Array used to address bitwise constants in the options word
//
UINT OptionConstants[NUM_OPTIONS] = { MINIFPORTCLOSED,
                                      TOPMOST,
                                      HIDECAPTION };                                      
                                       
                                                   
//==========================================================================
// Global variable declarations
//==========================================================================

char __far  glpszMLightsIniPath[60];    //Path to MLIGHTS.INI
HINSTANCE   hInst;             //Handle to the data segment of this instance
HWND        ghWnd;             //Handle to the main window
NPHOOKCHILD                             //Array of near pointers to hooked
npHookedFunctions[NUM_HOOKEDFUNCTIONS]; //function structures
CShape     Lights[NUM_LIGHTS];          //Modem light window objects
CLabel     LightLabels[NUM_LIGHTS];     //Modem light label objects
HFONT      ghLabelFont;      //Handle to the font to use for light labels
UINT       gEventWord;       //Comm device event word from GetCommEventMask() 
int        gPortID;          //Comm device ID returned from OpenComm()
BOOL       gbPortOpened;     //Flag indicating port is open or closed
UINT       gwOptions;        //Flag indicating the window's display options
UINT       gwNewOptions;     //Flag indicating a change in window's options
COMSTAT    gComStatBuf;      //Windows API structured variable containing 
                             //status information of a communications device
COLORREF   gdwLightColorOn;  //Color of lights when they are "ON"
COLORREF   gdwLightColorOff; //Color of lights when they are "OFF"
int        gnLeftPosition;   //Horizontal position of window
int        gnTopPosition;    //Vertical position of window
BOOL       gbRemoveCaptionOnRestore; //Flag for minimized pop up windows

//Specific registered messages that the Common Dialogs might send
UINT        iMSGColorOK;
UINT        iMSGSetRGB;


//==========================================================================
// Declarations for functions within this module
//==========================================================================

static void GetFilePath (LPSTR lpszFileName);
static void ReadIniValues ();
static void CreateWindowAccordingToOptions (int width, int clientheight);


//==========================================================================
// FUNCTION        WinMain
//
// PARAMETERS      hInstance      Handle to the data segment of this instance
//                 hPrevInstance  Handle to the data segment of the most 
//                                recent preceeding instance
//                 lpszCmdLine    Long pointer to the executable command line
//                 nCmdShow       Show window type (open or icon)
//
// RETURNS         Value of the PostQuitMessage function if the WinMain is
//                 successful.  NULL if the application is terminated before
//                 the main processing loop
//
// REMARKS         Main processing function for MLIGHTS.EXE
//==========================================================================
int PASCAL WinMain (HINSTANCE hInstance, 
                    HINSTANCE hPrevInstance,
                    LPSTR     lpszCmdLine, 
                    int       nCmdShow)
{
 MSG      msg;         //Message structured variable used in GetMessage loop
 BOOL     returncode;  //General variable to hold function return values

 //
 //If this is the first instance of the application, then
 //
 if (hPrevInstance == NULL)
    {
     //
     //Call the InitApplication function to initialize shared data
     //
     returncode = InitApplication (hInstance);
     
     //
     //If the InitApplication function failed, then
     //
     if (returncode == FALSE) 
        {
         //
         //Return false, thus aborting application
         //
         return (FALSE);     
        }
    }//End  if (hPrevInstance == NULL)

 //
 //Perform initializations that apply to this specific instance 
 //
 returncode = InitInstance (hInstance, nCmdShow);
 
 //
 //If the InitInstance function failed, then
 //
 if (returncode == FALSE)
    {
     //
     //Return false, thus aborting application
     //
     return (FALSE);
    }
     
 //     
 //Acquire and dispatch messages until a WM_QUIT message is received.
 //
 while (GetMessage (&msg,    //message structure 
                    NULL,    //handle of window receiving the message
                    NULL,    //lowest message to examine 
                    NULL))   //highest message to examine
    {
     TranslateMessage (&msg);    //Translates virtual key codes
     DispatchMessage (&msg);     //Dispatches message to window
    }
 return (msg.wParam);           //Returns the value from PostQuitMessage
} //End   WinMain ()


//==========================================================================
//
// FUNCTION: InitApplication(HANDLE)
//
// PURPOSE: Initializes window data and registers window class
//
// COMMENTS:
//
//     This function is called at initialization time only if no other
//     instances of the application are running.  This function performs
//     initialization tasks that can be done once for any number of running
//     instances.
//
//     In this case, we initialize a window class by filling out a data
//     structure of type WNDCLASS and calling the Windows RegisterClass()
//     function.  Since all instances of this application use the same window
//     class, we only need to do this when the first instance is initialized.
//
//==========================================================================

BOOL InitApplication (HINSTANCE hInstance)           
{
    /*
     * Register common dialog specific messages using strings defined in
     * COMMDLG.H.  Only the Help and Find messages are sent to our main
     * window procedure (the owner of the dialog box requesting help).
     * All others are sent to a dialog's hook procedure if one exists.
     */
    iMSGColorOK       =RegisterWindowMessage(COLOROKSTRING);  //Color
    iMSGSetRGB        =RegisterWindowMessage(SETRGBSTRING);   //Color

    WNDCLASS  wc;    //Window class stucture used to register a window class

    //
    //Fill in window class structure with parameters that describe the
    //main window
    //
    wc.style = CS_DBLCLKS;              //Class style(s)
    wc.lpfnWndProc = MainWndProc;       //Function to receive messages for
                                        //windows of this class
    wc.cbClsExtra = 0;                  //No per-class extra data
    wc.cbWndExtra = 0;                  //No per-window extra data
    wc.hInstance = hInstance;           //Application that owns the class
    wc.hIcon = LoadIcon (hInstance,     //Window class's icon
                         MAKEINTRESOURCE(IDI_LED));
    wc.hCursor = LoadCursor (NULL, IDC_ARROW);     //Window class's cursor 
    wc.hbrBackground = (HBRUSH) (COLOR_BTNFACE+1); //Window class's background
    wc.lpszMenuName =  NULL;                       //No default menu
    wc.lpszClassName = MAINWND_CLASSNAME;          //Class's name

    //
    //Register the window class and return success/failure code
    //
    return (RegisterClass (&wc));

}


//==========================================================================
//
// FUNCTION:  InitInstance(HANDLE, int)
//
// PURPOSE:  Saves instance handle and creates main window
//
// COMMENTS:
//
//     This function is called at initialization time for every instance of
//     this application.  This function performs initialization tasks that
//     cannot be shared by multiple instances.
//
//     In this case, we save the instance handle in a static variable and
//     create and display the main program window.
//
//==========================================================================

BOOL InitInstance (HINSTANCE hInstance, int nCmdShow)
{
 //
 //Register use of the Microsoft 3D controls
 //
 Ctl3dRegister (hInstance);
 Ctl3dAutoSubclass (hInstance);
 
 //
 //Read the MLIGHTS.INI values into global variables
 //
 ReadIniValues ();

 //
 // Save the instance handle in static variable, which will be used in
 // many subsequence calls from this application to Windows          
 //
 hInst = hInstance;


 //-------------------------------------------------------------------------
 // Below we create the main window 
 //-------------------------------------------------------------------------
 int width,                      //Width of window
     clientheight;               //Height of client area of window 
 
 //
 //The main window width will be 1/4 of the the screen width
 //
 width = GetSystemMetrics (SM_CXSCREEN) / 5;       //Find the width

 //
 //Height of client area only I have decided will be = caption height
 //
 clientheight = GetSystemMetrics (SM_CYCAPTION);     
 
 //
 //Call the static function to create the window with properties
 //according to the options word
 // 
 CreateWindowAccordingToOptions (width, clientheight);

 //
 // If window could not be created, return "failure" 
 //
 if (ghWnd == NULL) return (FALSE);

 //
 //Assign the child window regions which will emulate modem lights
 //
 CreateLights (width, clientheight);

 //
 //If the MINIFPORTCLOSED bit flag is on, then, 
 //
 if (gwOptions & MINIFPORTCLOSED)
    {
     //
     //Show the window minimized
     //
     ShowWindow (ghWnd, SW_SHOWMINNOACTIVE);  
     
     //
     //If this is a pop up window with HIDECAPTION 
     //
     if (gwOptions & HIDECAPTION)
        {
         //
         //Set our flag so we can remove the caption on window restore
         //
         gbRemoveCaptionOnRestore = TRUE;
        }//End  if (gwOptions & HIDECAPTION)
     else//Else  HIDECAPTION bit is not set
        {
         //
         //Set our flag indicating no need to remove caption on restore
         //
         gbRemoveCaptionOnRestore = FALSE;
        }//End else HIDECAPTION bit is not set
    }// End else MINIFPORTCLOSED bit flag is on
 else//MINIFPORTCLOSED bit flag is off
    {
     //
     //Show the window normally
     //
     ShowWindow (ghWnd, SW_SHOWNOACTIVATE);
    }//End  if (gwOptions & ! MINIFPORTCLOSED)

 return (TRUE);                 //Return success 
}


//==========================================================================
// FUNCTION        CreateWindowAccordingToOptions ()
//
// PARAMETERS      width         Width that the window is going to be
//                 clientheight  Height that the client area of the window
//                               shall be
//
// RETURNS         nothing
//
// REMARKS         Used only to call CreateWindowEx with the properties
//                 set according to the window's options word.  The options
//                 word is a WORD  of bit flags set according to the 
//                 entries in the MLIGHTS.INI file.
//                 Minimize if Port Closed
//                 Always on Top
//                 Hide Caption
//                 Previous window position
//==========================================================================
void CreateWindowAccordingToOptions (int width, int clientheight)
{
 int height;                     //Total height of window
 int left, top;                  //Left and top positions of the window
 DWORD dwStyle;                  //Regular window style
 DWORD dwExStyle;                //Extended window style

 //
 //Make height and style adjustments if the caption is hidden and 
 //window is not minimized
 //
 if ((gwOptions & HIDECAPTION) &&
    !(gwOptions & MINIFPORTCLOSED))
    {
     //
     //Height of client area only 
     //
     height = clientheight;
                     
     //
     //Adjust the window style
     //
     dwStyle = WS_POPUP | WS_BORDER;
    } 
 else    //Caption is shown
    {
     //
     //Height is client height (see above height assignment) 
     //plus caption height
     //
     height = clientheight + GetSystemMetrics (SM_CYCAPTION);
     
     //
     //Set the window style to have a caption, border, system menu, 
     //minimize box
     //
     dwStyle = WS_OVERLAPPED | WS_SYSMENU | WS_MINIMIZEBOX;
    }  //End else
 
 //
 //If the TOPMOST bit flag is on, AND, the MINIFPORTCLOSED flag if "Off" then, 
 //
 if ((gwOptions & TOPMOST) && (!(gwOptions & MINIFPORTCLOSED)))
    {
     //
     //Set the window extended style accordingly
     //
     dwExStyle = WS_EX_TOPMOST;
    } 
 else //Else the don't set the style to TOPMOST
    {
     //
     //Set the extended style to NULL
     //
     dwExStyle = NULL;
    } 
    
 //
 //If INI file had no settings, then global left and top positions will 
 //be -1.  In such a case, calculate new left and top positions
 //
 if (gnLeftPosition == -1 || gnTopPosition == -1)
    {
     //
     //Put window in bottom right of screen. Move top position up a little
     // (- GetSystemMetrics(SM_CYCAPTION)) out of the way of the Windows 95 
     //status bar
     //
     left = GetSystemMetrics(SM_CXSCREEN) - width;     
     top = GetSystemMetrics(SM_CYSCREEN) - height
           - GetSystemMetrics(SM_CYCAPTION);
    }  //End   if (gnLeftPosition == -1 || gnTopPosition == -1)
 else //If the INI file had settings for Left and top, then use them
    {
     left = gnLeftPosition; 
     top  = gnTopPosition;
    } //End else
      
 //
 //Create a main window for this application instance.   
 //
 ghWnd = CreateWindowEx(dwExStyle,                 //Extended style
                        MAINWND_CLASSNAME,         //Class name as registered
                        APP_TITLE,                 //Window name
                        dwStyle,                   //Window style
                        left, top, width, height,  //Position & dimensions
                        NULL,                      //No parent 
                        NULL,                      //No menu
                        hInst,                     //This instance handle
                        NULL);                     //No extra parameters

} //End  void CreateWindowAccordingToOptions (int width, int clientheight)


//==========================================================================
// FUNCTION        CreateLights ()
//
// PARAMETERS      width      Width of the window enclosing the lights
//                 height     Height of the window enclosing the lights
//
// RETURNS         nothing
//
// REMARKS         Used to assign values to the array containing the 
//                 positions of the child windows which will emulate 
//                 modem lights.
//==========================================================================

void CreateLights (int width, int height)
{
 //-------------------------------------------------------------------------
 //Create the dimensions of one light, calculated based on size of window
 //-------------------------------------------------------------------------
 //
 //Light Diameter:
 //      1. After fiddling, I got an appropriate height and width of a light.
 //
 int lightheight = height / 2 - 2;
 int lightwidth = lightheight * 1.5 ;     

 //
 //Light Left postion:
 //      1. Take the entire width of the main window and chop it into
 //         NUM_LIGHTS equal region. 
 //
 int lightregion = width / NUM_LIGHTS;
 
 //
 //      2. So, centre the light within the region by subtracting the 
 //         light diameter from the width of the region and dividing by two
 int lightleft = (lightregion - lightwidth) / 2 ;
 
 //Light Top position:
 //      1. Determine how much smaller the light height will be 
 //         than the window height
 //      2. The light top position will be one third the result above in 1.
 //         This will allow space for the light labels under each light
 //
 int lighttop = 2; 
 
 //
 //Now we create our first light
 //
 CShape Light (lightleft,             //Left edge distance from left of screen
               lighttop,              //Top edge distance from top of screen
               lightleft + lightwidth, //Right edge distance from screen left
               lighttop + lightheight, //Bottom edge distance from screen top
               gdwLightColorOn,        //Color of light when "ON"
               gdwLightColorOff);      //Color of light when "OFF"

 //
 //Use the one light created above to initialize an array of Lights
 //
 for (int iloop = 0; iloop < NUM_LIGHTS; iloop++)
    {
     Lights[iloop] = Light;                  //Assign position
     Light.Offset (lightregion, 0);          //Shift right 
    }               
 
 
 //-------------------------------------------------------------------------
 // Now we create the text labels for each light
 //-------------------------------------------------------------------------
 //
 //Create the font to use to display labels
 //
 ghLabelFont = CreateFont(height / 2,         //Height of font
                          0, 
                          0, 
                          0, 
                          FW_BOLD, 
                          0, 
                          0, 
                          0,
                          ANSI_CHARSET, 
                          OUT_DEFAULT_PRECIS, 
                          CLIP_DEFAULT_PRECIS,
                          DEFAULT_QUALITY, 
                          VARIABLE_PITCH | FF_SWISS, 
                          "Serif");  

 //
 //Insert the new font into the device context for the main window
 //
 HDC hdc = GetDC (ghWnd); 
 HFONT hOldFont = SelectObject (hdc, ghLabelFont);
 
 //
 //Remember the top position at which each light label will appear
 //The -1 in the calculation is just the product of fiddling about trying
 //to get the display to look pretty.
 //
 int labeltop = lighttop + lightheight - 1; 

 int labelleft;     //Left position of the label
 long nLabelLength; //Length of the label text using this font

 //
 //Assign the character string to each label
 //
 for (iloop = 0; iloop < NUM_LIGHTS; iloop++)
    {
     //
     //Assign the text string for the label
     //
     LightLabels[iloop].SetLabelString (LabelConstants[iloop]);

     //
     //Determine how long this string is using this font in 
     //this device context
     //
     nLabelLength = GetTextExtent (hdc, LabelConstants[iloop],
                                   lstrlen (LabelConstants[iloop]));
                                   
     //
     //In order to centre the label below the light, subtract the
     //length of the label from the width of the region, and divide by two
     //
     labelleft = (iloop * lightregion) + 
                 (lightregion - (int) nLabelLength) / 2;

     //
     //Assign the left and top position of the label 
     //
     LightLabels[iloop].SetPosition (labelleft, labeltop);
    } 
 
 //
 //Put the old font back into the device context, 
 //and release the device context
 //
 SelectObject (hdc, hOldFont);
 ReleaseDC (ghWnd, hdc);   
 
} //End  CreateLights ()


//==========================================================================
// FUNCTION        ReadIniValues ()
//
// PARAMETERS      none
//
// RETURNS         nothing
//
// REMARKS         Reads the settings in the MLIGHTS.INI file. The global
//                 variable for the MLIGHTS.INI path will be assigned
//                 in this function.
//==========================================================================
void ReadIniValues ()
{
 char szCaptionSetting[5];
 
 //
 //Start with a clear word of options bit flags
 //
 gwOptions = 0x0000;
 
 //
 //Retrieve the path to the MLIGHTS.INI file into a global for it
 //
 lstrcpy (glpszMLightsIniPath, INI_FILENAME);
 GetFilePath (glpszMLightsIniPath); 

 for (int iloop = 0; iloop < NUM_OPTIONS; iloop++)
    {
     //
     //Read the option setting
     //
     GetPrivateProfileString ("Options",
                              OptionEntryStrings[iloop],
                              "Off",
                              szCaptionSetting,
                              sizeof (szCaptionSetting),
                              glpszMLightsIniPath);
                              
     //
     //If the option is "On" in MLIGHTS.INI, then,
     //
     if (_fstricmp (szCaptionSetting, "On") == 0) 
        {
         //
         //Set the bit flag in the options word
         //
         gwOptions = gwOptions | OptionConstants[iloop];
        }
    } //End  for (int iloop = 0; iloop < NUM_OPTIONS; iloop++)                                      

 //
 //Read the color for the lights when they are "ON". Default is RED.
 //The color is stored in MLIGHTS.INI as a string which must be 
 //converted to a DWORD (unsigned long).
 //
 char colorvalue[15];
 GetPrivateProfileString ("Colors",             //[Colors] section
                          "ColorOn",            //"ColorOn" entry
                          "255",                //Default value
                          colorvalue,           //Pointer to string buffer
                          15,                   //Size of string buffer
                          glpszMLightsIniPath); //[path]MLIGHTS.INI 
 //
 //Convert the colorvalue string to a DWORD
 //
 gdwLightColorOn = strtoul (colorvalue, NULL, 10);                         
 
 //
 //Read the color for the lights when they are "OFF". Default is DARK RED.
 //The color is stored in MLIGHTS.INI as a string which must be 
 //converted to a DWORD (unsigned long).
 //
 GetPrivateProfileString ("Colors",             //[Colors] section
                          "ColorOff",           //"ColorOff" entry
                          "127",                //Default value
                          colorvalue,           //Pointer to string buffer
                          15,                   //Size of string buffer
                          glpszMLightsIniPath); //[path]MLIGHTS.INI 
 //
 //Convert the colorvalue string to a DWORD
 //
 gdwLightColorOff = strtoul (colorvalue, NULL, 10);                         
 
 //
 //Read the left position of the window
 //
 char coordinatevalue[15];
 GetPrivateProfileString ("Position",             //[Position] section
                          "Left",                 //"Left" entry
                          "-1",                   //Default value
                          coordinatevalue,        //Pointer to string buffer
                          15,                     //Size of string buffer
                          glpszMLightsIniPath);   //[path]MLIGHTS.INI 
 //
 //Convert the coordinatevalue string to a word
 //
 gnLeftPosition = atoi (coordinatevalue);

 //
 //Read the top position of the window
 //
 GetPrivateProfileString ("Position",             //[Position] section
                          "Top",                  //"Top" entry
                          "-1",                   //Default value
                          coordinatevalue,        //Pointer to string buffer
                          15,                     //Size of string buffer
                          glpszMLightsIniPath);   //[path]MLIGHTS.INI 
 //
 //Convert the coordinatevalue string to a word
 //
 gnTopPosition = atoi (coordinatevalue);
 
} //End  void ReadIniValues () 


//==========================================================================
// FUNCTION        GetFilePath
//               
// PARAMETERS      lpszFileName    Pointer to the character string of the
//                                 file name.
//
// RETURNS         void            lpszFileName will contain the full path
//                                 and file name if successful or will be
//                                 unchanged if unsuccessful.
//
// REMARKS         Used to determine the full path name of a file. If 
//                 no file exists, function returns lpszFileName unchanged.
//=========================================================================
void GetFilePath (LPSTR lpszFileName)
{
 int xvalue;                     //General purpose function return value
 OFSTRUCT FAR* lpOpenBuffer;     //Structure to hold file information
    
 lpOpenBuffer = new OFSTRUCT;         //Allocate the file info buffer
    
 xvalue = OpenFile(lpszFileName, lpOpenBuffer, OF_EXIST); //Check for file
    
 if (xvalue != HFILE_ERROR)                     //If OpenFile succeeded
    {
     _fstrcpy (lpszFileName, lpOpenBuffer->szPathName); 
    }

 delete lpOpenBuffer;                       //Free the file info buffer
}  //End  void GetFilePath (LPCSTR lpszFileName)

                 